#include <stm32f0xx.h>
#include <gpio.hpp>
using namespace stm32f072xb;
//https://www.st.com/resource/en/reference_manual/dm00031936-stm32f0x1stm32f0x2stm32f0x8-advanced-armbased-32bit-mcus-stmicroelectronics.pdf

void enableGpio()
{
	// Gpio gpio = Gpio();
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN); //enable GPIOA (p.120)
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOCEN); //enable GPIOC (p.120)
}

void setupButton(uint32_t buttonPin)
{
	//button
	CLEAR_BIT(GPIOC->MODER, (0x3 << buttonPin * 2)); //input mode (p.157)
	CLEAR_BIT(GPIOC->PUPDR, (0x3 << buttonPin * 2));
	GPIOC->PUPDR |= (0x1 << (buttonPin * 2)); //pull-up (p.157)
}

void setupLed(const uint32_t ledPin)
{
	//LED
	CLEAR_BIT(GPIOA->MODER, (0x3 << ledPin * 2));
	GPIOA->MODER |= (0x1 << (ledPin * 2));	 //Output mode (p.157)
	CLEAR_BIT(GPIOA->OTYPER, (1 << ledPin)); // Push-pull output (p.157)
}

bool isPressed(const uint32_t buttonPin)
{
	return !READ_BIT(GPIOC->IDR, 1 << buttonPin); //read button p.159
}
int main()
{
	enableGpio();

	constexpr uint32_t buttonPin = 13;
	setupButton(buttonPin);

	constexpr uint32_t ledPin = 5;
	setupLed(ledPin);

	while (true)
	{
		if (isPressed(buttonPin))
		{
			SET_BIT(GPIOA->ODR, 1 << ledPin); //turn on LED, p.159
		}
		else
		{
			CLEAR_BIT(GPIOA->ODR, 1 << ledPin); //turn off LED, p.159
		}
	}

	return 0;
}
