#include <stm32f0xx.h>
#include <gpio.hpp>

using namespace stm32f072xb;

extern "C" void TIM6_DAC_IRQHandler()
{
	SET_BIT(GPIOA->ODR, GPIO_ODR_5); // Turn on LED, p.159
	CLEAR_BIT(TIM6->SR, TIM_SR_UIF); // TIM6 status register :: Clear interrupt (p. 557)
}

void setupGpio()
{
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN); // Enable GPIOA (p. 120)
}

void setupLed()
{
	CLEAR_BIT(GPIOA->MODER, GPIO_MODER_MODER5);
	SET_BIT(GPIOA->MODER, GPIO_MODER_MODER5_0); // Output mode (p.157)
	CLEAR_BIT(GPIOA->OTYPER, GPIO_OTYPER_OT_5); // Push-pull output (p.157)
	CLEAR_BIT(GPIOA->ODR, GPIO_ODR_5);			// Turn off LED, p.159
}

void setupTim6()
{
	constexpr uint16_t autoReloadValue = 122 - 1;	// 8 000 000 / 65535 = 122.07 clk cycles to burn
	constexpr uint16_t prescalerValue = 0xFFFF - 1; // clk / 65535

	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_TIM6EN); // APB peripheral clock enable register 1 :: TIM6 enable clock (p. 123)
	WRITE_REG(TIM6->ARR, autoReloadValue);	   // TIM6 auto-reload register :: Value to reach for counter (p. 558)
	WRITE_REG(TIM6->PSC, prescalerValue);	   // TIM6 prescaler (p. 558)
	SET_BIT(TIM6->EGR, TIM_EGR_UG);			   // Update preload value (ARR and PSC) (p. 557) and p.549 for diagram example
	CLEAR_BIT(TIM6->SR, TIM_SR_UIF);		   // Clear the interrupt status flag (p. 557)
	SET_BIT(TIM6->CR1, TIM_CR1_OPM);		   // One pulse mode (p. 555)
}

void setupInterrupts()
{
	constexpr uint16_t interruptPriority = 0;

	SET_BIT(TIM6->DIER, TIM_DIER_UIE);					// TIM6 Interrupt enable register :: Enable update interrupt (p. 556)
	NVIC_EnableIRQ(TIM6_DAC_IRQn);						// Enable interrupts for TIM6 (p. 936)
	NVIC_SetPriority(TIM6_DAC_IRQn, interruptPriority); // Sets interrupts' priority to the highest (0)
}

void startTim6Counter()
{
	SET_BIT(TIM6->CR1, TIM_CR1_CEN); // TIM6 ctrl register 1 :: Counter enable (p. 555)
}

int main()
{
	setupGpio();
	setupLed();
	setupTim6();
	setupInterrupts();
	startTim6Counter();

	while (true)
		;
}
