#include <stm32f0xx.h>
#include <gpio.hpp>
using namespace stm32f072xb;
// https://www.st.com/resource/en/reference_manual/dm00031936-stm32f0x1stm32f0x2stm32f0x8-advanced-armbased-32bit-mcus-stmicroelectronics.pdf
// https://www.st.com/resource/en/datasheet/stm32f072c8.pdf

void setupUART()
{
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART2EN); // enable uart1 (p.116)
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN);	 //enable GPIOA for the uart1 (p.120)

	CLEAR_BIT(GPIOA->MODER, GPIO_MODER_MODER2);
	SET_BIT(GPIOA->MODER, GPIO_MODER_MODER2_1);		 //alternate mode (p.157)
	GPIOA->AFR[0] = (GPIO_AFRL_AFSEL2 & 0x00000100); // enable UART function for PA2-Tx (p.162)
	CLEAR_BIT(GPIOA->OTYPER, GPIO_OTYPER_OT_2);		 //push-pull mode

	// NE PAS DECOMMENTER LE BLOC EN BAS

	// CLEAR_BIT(GPIOA->MODER, GPIO_MODER_MODER3);
	// SET_BIT(GPIOA->MODER, GPIO_MODER_MODER3_1);		 //alternate mode (p.157) ok
	// GPIOA->AFR[0] = (GPIO_AFRL_AFSEL3 & 0x00001000); // enable UART function for PA3-Rx (p.162)
	// CLEAR_BIT(GPIOA->OTYPER, GPIO_OTYPER_OT_3);		 //push-pull mode

	WRITE_REG(USART2->CR1, 0x0);
	CLEAR_BIT(USART2->CR1, USART_CR1_M0 | USART_CR1_M1);		 // using 8bits for words (p.732)
	CLEAR_BIT(USART2->CR2, USART_CR2_STOP_0 | USART_CR2_STOP_1); // using 1 stop bit (p.737)
	SET_BIT(USART2->CR1, USART_CR1_UE);							 // activate USART2
	SET_BIT(USART2->CR1, USART_CR1_RXNEIE);
	WRITE_REG(USART2->BRR, 833); //using the baud rate equation at p.703 and the baud rate register p.743
}

void enableTransmission()
{

	SET_BIT(USART2->CR1, USART_CR1_TE); //enable transmitter (p.734)
}

void enableReception()
{

	SET_BIT(USART2->CR1, USART_CR1_RE); //enable Reception (p.734)
}

char readWord()
{
	while (!(USART2->ISR & USART_ISR_RXNE))
		;
	return (uint8_t)(USART2->RDR);
}
void writeWord(uint8_t word)
{
	while (!((USART_ISR_TXE & USART2->ISR)))
		;
	USART2->TDR = (char)(word);
	while (!(USART_ISR_TC & USART2->ISR))
		;
}

int main()
{
	setupUART();
	enableTransmission();
	enableReception();
	uint8_t word;

	while (true)
	{
		word = readWord();
		writeWord(word);
	}
	return 0;
}
